local addonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables
local c = BittensGlobalTables.GetTable("BittensSpellFlashLibrary")

local min = math.min
local max = math.max

------------------------------------------------------------------------ Common
c.AddSpell("Defensive Stance", nil, {
   Type = "form"
})

c.AddSpell("Battle Stance", nil, {
   Type = "form"
})

c.AddSpell("Battle Shout", nil, {
   NoRangeCheck = true,
   CheckFirst = function(z)
      if s.InRaidOrParty() then
         c.MakeOptional(z, false)
         return c.RaidBuffNeeded(c.ATTACK_POWER_BUFFS)
      else
         c.MakeOptional(z, c.HasMyBuff("Commanding Shout"))
         return c.SelfBuffNeeded(c.ATTACK_POWER_BUFFS)
      end
   end
})

c.AddSpell("Commanding Shout", nil, {
   NoRangeCheck = true,
   CheckFirst = function(z)
      if s.InRaidOrParty() then
         c.MakeOptional(z, c.RaidBuffNeeded(c.ATTACK_POWER_BUFFS))
         return c.RaidBuffNeeded(c.STAMINA_BUFFS)
      else
         return false
      end
   end
})

c.AddOptionalSpell("Charge")

c.AddSpell("Bladestorm", nil, {
   Melee = true,
   Cooldown = 60,
})

c.AddSpell("Bladestorm", "if Enraged", {
   Melee = true,
   Cooldown = 60,
   CheckFirst = function()
      return a.Enraged
   end
})

c.AddSpell("Bladestorm", "if Enraged for duration", {
   Melee = true,
   Cooldown = 60,
   CheckFirst = function()
      return a.Enraged and c.GetBuffDuration("Enrage") >= 6
   end
})

c.AddSpell("Bladestorm", "with Colossus Smash", {
   Melee = true,
   Cooldown = 60,
   CheckFirst = function()
      return c.HasMyDebuff("Colossus Smash")
         and a.Rage < 70
   end
})

c.AddSpell("Shockwave", nil, {
   CheckFirst = function()
       return not c.HasTalent("Unquenchable Thirst")
   end
})

-- c.MakeMini(c.AddOptionalSpell("Heroic Leap", nil, {
--    NoGCD = true,
-- --   CheckFirst = function()
-- --      return a.Smash > 0 and not c.IsSolo()
-- --   end
-- }))

local function victoryHealable()
   local heal = 15
   if a.VictoriousDuration > 4 then
      if c.HasGlyph("Victory Rush") and not c.HasTalent("Impending Victory") then
         heal = heal * 1.5
      end
   end

   return c.GetHealthPercent("player") + heal < (c.IsSolo() and 102 or 70)
end

c.AddSpell("Impending Victory", nil, {
   Rage = 10,
   Cooldown = 30,
   CheckFirst = function()
      return c.GetHealthPercent("target") > 20
         and not c.HasTalent("Unquenchable Thirst")
   end
})

c.AddSpell("Impending Victory", "for Heals", {
   Rage = 10,
   Cooldown = 30,
   CheckFirst = victoryHealable,
})

c.AddSpell("Victory Rush", "for Heals", {
   Melee = true,
   CheckFirst = function()
      return a.VictoriousDuration > 0 and victoryHealable()
   end,
})

c.AddOptionalSpell("Avatar", nil, {
   NoGCD = true,
   Cooldown = 3 * 60,
   CheckFirst = function()
      return c.HasBuff("Recklessness")
         or c.GetHealthPercent("target") < 5
   end
})

c.AddOptionalSpell("Siegebreaker", nil, {
   Cooldown = 45
})

c.AddOptionalSpell("Storm Bolt", nil, {
   Cooldown = 30,
})

c.AddOptionalSpell("Dragon Roar", nil, {
   Melee = true,
   Cooldown = 60,
   CheckFirst = function()
      return a.Bloodbath or not c.HasTalent("Bloodbath")
   end
})

c.AddSpell("Execute", nil, {
   Rage = function()
      return s.Spec(1) and 10 or 30
   end,
   CheckFirst = function()
      return a.InExecute
   end
})

c.AddSpell("Execute", "with Sudden Death", {
   CheckFirst = function()
      return a.SuddenDeath
   end
})

c.AddSpell("Whirlwind", nil, {
   Melee = true,
   Rage = 20,
})

c.AddSpell("Whirlwind", "near cap", {
   Melee = true,
   Rage = 20,
   CheckFirst = function()
      return a.EmptyRage <= 20
   end
})


c.AddInterrupt("Pummel")
c.AddInterrupt("Disrupting Shout")

-------------------------------------------------------------------------- Arms
c.AddSpell("Execute", "with Colossus Smash", {
   Rage = 10,
   CheckFirst = function()
      if not a.InExecute then
         return false
      end

      if c.HasMyDebuff("Colossus Smash") then
         return true
      end

      if c.GetHealthPercent("target") < (s.Boss() and 2 or 20) then
         return true
      end

      return (a.Rage > 60 or c.EstimatedHarmTargets == 2)
         and c.GetCooldown("Colossus Smash") >= c.LastGCD
   end
})

c.AddSpell("Mortal Strike", nil, {
   Rage = 20,
   Cooldown = 6,
   CheckFirst = function()
      return c.GetCooldown("Colossus Smash") > 1
         and not a.InExecute
   end
})

c.AddSpell("Mortal Strike", "with two targets", {
   Rage = 20,
   Cooldown = 6,
   CheckFirst = function()
      return c.GetCooldown("Colossus Smash") > 1.5
         and not a.InExecute
         and c.EstimatedHarmTargets == 2
   end
})

c.AddSpell("Rend", nil, {
   Rage = 5,
   MyDebuff = "Rend",
   CheckFirst = function()
      return not c.HasMyDebuff("Rend")
   end
})

c.AssociateTravelTimes(0, "Rend")
c.AddSpell("Rend", "early", {
   Rage = 5,
   MyDebuff = "Rend",
   CheckFirst = function()
      return c.ShouldCastToRefresh("Rend", "Rend", 6, true)
   end
})

c.AddSpell("Rend", "early without Colossus Smash", {
   Rage = 5,
   MyDebuff = "Rend",
   CheckFirst = function()
      return c.ShouldCastToRefresh("Rend", "Rend", 5.4, true)
         and not c.HasMyDebuff("Colossus Smash")
   end
})

c.AddOptionalSpell("Recklessness", "for Arms", {
   Cooldown = 3 * 60,
   CheckFirst = function()
      local hp = c.GetHealthPercent("target")
      local boss = s.Boss("target")

      -- if target.time_to_die<10 then return true

      if not c.HasMyDebuff("Rend") then
         return false
      end

      if hp > 20 and not boss then
         return false
      end

      if c.HasTalent("Bloodbath") then
         return a.Bloodbath
      elseif c.GetMyDebuffDuration("Colossus Smash") >= 5 then
         return 0
      else
         return c.GetCooldown("Colossus Smash") - 2, 0.35
      end
   end
})

c.AddOptionalSpell("Bloodbath", "for Arms", {
   NoGCD = true,
   Cooldown = 60,
   CheckFirst = function()
      return c.HasMyDebuff("Rend")
         and c.GetCooldown("Colossus Smash") < 5
   end
})

c.AddOptionalSpell("Sweeping Strikes", nil, {
   NoGCD = true,
   Rage = 10,
})

c.AddSpell("Dragon Roar", "for Arms", {
   Melee = true,
   Cooldown = 60,
   CheckFirst = function()
      return c.GetCooldown("Colossus Smash") > c.LastGCD
         and not c.HasMyDebuff("Colossus Smash")
   end
})

-- @todo danielp 2015-01-22: not sure this actually makes sense, but there was
-- definitely existing code tracking it, so maybe we should assume it sticks?
--
-- should test, once >= 81, if this is a buff, debuff, and on what, and if
-- there is a travel time delay between the cast and the buff.
c.AssociateTravelTimes(0, "Colossus Smash")
c.AddSpell("Colossus Smash", nil, {
   CheckFirst = function()
      return not c.IsCastingOrInAir("Colossus Smash")
   end
})

c.AddSpell("Colossus Smash", "with Rend", {
   CheckFirst = function()
      return not c.IsCastingOrInAir("Colossus Smash")
         and c.HasMyDebuff("Rend")
   end
})

c.AddSpell("Slam", nil, {
   Rage = function()
      return 10 + (c.GetBuffStack("Slam") * 10)
   end,
   CheckFirst = function()
      return (a.Rage > 20 or c.GetCooldown("Colossus Smash") > c.LastGCD)
         and not a.InExecute
         and c.GetCooldown("Colossus Smash") > 1
         and c.GetCooldown("Mortal Strike") > 1
   end
})

c.AddSpell("Whirlwind", "for Arms ST", {
   Melee = true,
   Rage = 20,
   CheckFirst = function()
      return not c.HasTalent("Slam")
         and not a.InExecute
         and (a.Rage >= 40 or c.WearingSet(4, "T17") or c.HasMyDebuff("Colossus Smash"))
         and c.GetCooldown("Colossus Smash") > 1
         and c.GetCooldown("Mortal Strike") > 1
   end,
})

c.AddSpell("Whirlwind", "for Arms AoE", {
   Melee = true,
   Rage = 20,
   CheckFirst = function()
      return c.GetCooldown("Colossus Smash") > 1.5
         and (a.InExecute or c.EstimatedHarmTargets > 3)
   end,
})

c.AddOptionalSpell("Ravager", "with Colossus Smash", {
   Cooldown = 60,
   GetDelay = function()
      -- sync with colossus smash
      return max(c.GetCooldown("Colossus Smash") - 4,
                 c.GetCooldown("Ravager"))
   end
})

c.AddOptionalSpell("Ravager", "sync with Bloodbath", {
   Cooldown = 60,
   GetDelay = function()
      if c.HasTalent("Bloodbath") then
         return max(c.GetCooldown("Bloodbath"), c.GetCooldown("Ravager"))
      else
         return c.GetCooldown("Ravager")
      end
   end
})

c.AddOptionalSpell("Storm Bolt", "with Colossus Smash", {
   Cooldown = 30,
   CheckFirst = function()
      if a.Rage >= 90 then
         return false
      end

      return c.HasMyDebuff("Colossus Smash")
         or c.GetCooldown("Colossus Smash") > 4
   end
})

c.AddSpell("Impending Victory", "for Arms", {
   Rage = 10,
   Cooldown = 30,
   CheckFirst = function()
      return a.Rage < 40
         and not a.InExecute
         and c.GetCooldown("Colossus Smash") > 1
         and c.GetCooldown("Mortal Strike") > 1
   end
})

------------------------------------------------------------------------- Fury

local function getRagingBlowStacks(noGCD)
   local stacks = c.GetBuffStack("Raging Blow!", noGCD)
   if c.IsCasting("Enrage") then
      stacks = stacks + 1
   end
   if c.IsCasting("Raging Blow") then
      stacks = stacks - 1
   end
--   local str = tostring(min(max(0, stacks), 2))
   return min(max(0, stacks), 2)
end

c.AddOptionalSpell("Berserker Rage", "for Fury", {
   NoGCD = true,
   Cooldown = 30,
   CheckFirst = function()
      return not a.Enraged
         or (c.HasTalent("Unquenchable Thirst") and getRagingBlowStacks(true) < 0)
   end
})

c.AddOptionalSpell("Bloodbath", nil, {
   NoGCD = true,
   Cooldown = 60
})


c.AddSpell("Bloodthirst", "early", {
   Cooldown = 4.5,
   CheckFirst = function()
      return (a.Rage < 80 and not c.HasTalent("Unquenchable Thirst"))
         or not a.Enraged
   end
})

c.AddSpell("Bloodthirst", nil, {
   Cooldown = 4.5,
})

c.AddSpell("Bloodthirst", "during cleave", {
   Cooldown = 4.5,
   CheckFirst = function()
      return not a.Enraged
         or getRagingBlowStacks() <= 0
         or a.Rage < 50
   end
})

c.AddSpell("Execute", "when Enraged", {
   Rage = 30,
   CheckFirst = function()
      -- @todo danielp 2015-01-18: actually TTD < 12
      return a.InExecute and (a.Enraged or c.GetHealthPercent("target") < 2)
   end
})

-- Throw your weapon at the enemy, causing 50% Physical damage.   Generates high threat.
-- Cast is instant with a 6 sec. cooldown.   Range is 8-30 yards
c.AddOptionalSpell("Heroic Throw", "for Fury", {
--   NoRangeCheck = false,
--   CheckFirst = function()
--      return a.EmptyRage <= 10
--         or (not a.InExecute and a.Rage >= 40 and a.smashPending > 0)
--   end
})

c.AddSpell("Raging Blow", nil, {
   Rage = 10,
   CheckFirst = function()
      -- Blizzard, why couldn't this just be normal spell charges?
      return getRagingBlowStacks() > 0
   end
})

c.AddSpell("Raging Blow", "with Meat Cleaver", {
   Rage = 10,
   CheckFirst = function()
      -- Blizzard, why couldn't this just be normal spell charges?
      return getRagingBlowStacks() > 0
         and c.GetBuffStack("Meat Cleaver") > 0
   end
})

c.AddSpell("Raging Blow", "with 2x Meat Cleaver", {
   Rage = 10,
   CheckFirst = function()
      -- Blizzard, why couldn't this just be normal spell charges?
      return getRagingBlowStacks() > 0
         and c.GetBuffStack("Meat Cleaver") >= 2
   end
})

c.AddSpell("Raging Blow", "with 3x Meat Cleaver", {
   Rage = 10,
   CheckFirst = function()
      -- Blizzard, why couldn't this just be normal spell charges?
      return getRagingBlowStacks() > 0
         and c.GetBuffStack("Meat Cleaver") >= 3
   end
})
c.AddSpell("Raging Blow", "with 3x Meat Cleaver and Enraged", {
   Rage = 10,
   CheckFirst = function()
      -- Blizzard, why couldn't this just be normal spell charges?
      return getRagingBlowStacks() > 0
         and c.GetBuffStack("Meat Cleaver") >= 3
         and a.Enraged
   end
})

c.AddOptionalSpell("Recklessness", "for Fury", {
   NoGCD = true,
   Cooldown = 3 * 60,
   CheckFirst = function()
      local hp = c.GetHealthPercent("target")

      -- @todo danielp 2015-01-18: should be "or TTD <= 12"
      if c.HasTalent("Anger Management") or hp <= 5 then
         return true
      end

      -- @todo danielp 2015-01-18: should be "TTD >= 190 or hp <= 20" here.
      return (s.Boss("target") or hp <= 20)
         and (a.Bloodbath or not c.HasTalent("Bloodbath"))
   end
})

c.AddSpell("Shockwave", "for Fury", {
   CheckFirst = function()
       return not c.HasTalent("Unquenchable Thirst")
   end
})

c.AddSpell("Whirlwind", "for Meat Cleaver", {
   Melee = true,
   Rage = 20,
   CheckFirst = function()
      return not c.HasBuff("Meat Cleaver")
   end
})

local function wildStrikeCost()
   if c.HasBuff("Bloodsurge") then
      return 0
   elseif c.HasTalent("Furious Strikes") then
      return 20
   else
      return 45
   end
end

c.AddSpell("Wild Strike", "at Cap", {
   Rage = wildStrikeCost,
   SpecialGCD = 1,
   CheckFirst = function()
      return a.EmptyRage <= 10 and s.HealthPercent() > 20
   end
})

-- Bloodsurge is up so cost is zero, fire away ...
c.AddSpell("Wild Strike", "with Bloodsurge", {
   Rage = wildStrikeCost,
   SpecialGCD = 1,
   CheckFirst = function()
      return c.HasBuff("Bloodsurge")
   end
})

c.AddSpell("Wild Strike", "if Enraged", {
   Rage = wildStrikeCost,
   SpecialGCD = 1,
   CheckFirst = function()
      return a.Enraged
   end
})

c.AddSpell("Wild Strike", "with Bloodsurge to avoid cap", {
   Rage = wildStrikeCost,
   SpecialGCD = 1,
   CheckFirst = function()
      return c.HasBuff("Bloodsurge")
         and a.Rage > 75
   end
})

c.AddOptionalSpell("Ravager", "for Fury", {
   Cooldown = 60,
   GetDelay = function()
      -- delay until BB comes off cooldown, to sync them, unless that is more
      -- than ten seconds wait.  given they are both on a one minute CD, most
      -- people will likely macro the two and all, so we def. want this to
      -- flash as soon as it is usable, since they may not even have BB on
      -- their bars to flash at them, just macro'ed into this.
      local cd = c.GetCooldown("Bloodbath")
      if a.Bloodbath or cd > 10 or c.GetHealthPercent("target") < 5 then
         return 0
      else
         return cd
      end
   end
})

-------------------------------------------------------------------- Protection
c.AddOptionalSpell("Berserker Rage", "for Prot", {
   NoGCD = true,
   CheckFirst = function()
      return (c.IsTanking() or c.InDamageMode()) and c.GetCooldown("Berserker Rage") == 0
   end,
})

--   For the next 12 sec, your melee damage abilities and their multistrikes
-- deal 30% additional   damage as a bleed over 6 sec. While bleeding,
-- targets move 50% slower.   Has a 60 sec Cooldown
c.AddOptionalSpell("Bloodbath", "for Prot", {
   NoGCD = true,
   Cooldown = 60,
   CheckFirst = function()
       return c.GetCooldown("Bloodbath")   == 0
   end,
})

c.AddSpell("No Mitigation if Victory Available", nil, {
   ID = 0,
   RunFirst = function(z)
      z.ID = c.HasTalent("Impending Victory")
         and c.GetID("Impending Victory")
         or c.GetID("Victory Rush")
   end,
   ShouldHold = function()
      return true
   end
})

--10122014 rdh: Added Cooldown
--Demoralizes all enemies within 10 yards, reducing the
--damage they do to you by 20% for 8 sec.
c.AddOptionalSpell("Demoralizing Shout", "for Prot", {
   NoGCD = true,
   Melee = true,
   IsUp = function()
      return c.HasMyDebuff("Demoralizing Shout", true)
   end,
   CheckFirst = function(z)
      return not z.IsUp()
   end,
})

-- Deals 200% Physical damage, and has a 30% chance to reset the cooldown on
-- Shield Slam and cause it to generate 5 more Rage.   GCD is 1.5 sec.
c.AddSpell("Devastate", "for Prot", {
   Melee = true,
   CheckFirst = function()
       return c.GetBuffStack("Unyielding Strikes") <= 5
   end,
})

-- Deals (165% of Attack Power) damage to all enemies within 8 yards and knocking
-- them back and down for 0.5 sec.   Damage ignores all armor and is
-- always a critical strike.
-- Cooldown is 1 min. and is an instant cast, no cost with GCD of 1.5 sec.
c.AddSpell("Dragon Roar", "for Prot", {
   Melee = true,
   CheckFirst = function()
      return c.HasTalent("Dragon Roar") and c.GetCooldown("Dragon Roar") == 0
   end
})

-- Instantly heals for 10% of your maximum health, and an additional
-- 20% over 5 sec.   Usable while stunned.
c.AddOptionalSpell("Enraged Regeneration", "for Prot", {
   NoGCD = true,
--   CheckFirst = function()
--      return a.Enraged
--   end,
   ShouldHold = function()
      return c.GetHealthPercent("player") < 50
   end,
})

-- Attempt to finish off a wounded foe, causing
-- 3,237 Physical damage with main-hand and 969
-- Physical damage with off-hand.   Only usable on
-- enemies with less than 20% health.
c.AddSpell("Execute", "for Prot", {
   CheckFirst = function()
      return a.SuddenDeath or a.InExecute
   end
})

-- Instantly deals 105% Physical damage.   This ability is not on the global cooldown.
-- Costs 30 rage with 1.5 sec cooldown
c.AddSpell("Heroic Strike", "for Prot", {
   Continue = false,
   NotIfActive = true,
   CheckFirst = function()
      return c.HasBuff("Ultimatum")
         or (c.HasTalent("Unyielding Strikes") and c.GetBuffStack("Unyielding Strikes") >= 5)
          or a.Rage >= a.maxRage - 30
   end,
})

-- Throw your weapon at the enemy, causing 50% Physical damage.   Generates high threat.
-- Cast is instant with a 6 sec. cooldown.   Range is 8-30 yards
c.AddOptionalSpell("Heroic Throw", "for Prot", {
   EnemyTargetNeeded = true,
   CheckFirst = function()
      return c.GetCooldown("Heroic Throw") == 0
   end,
})

-- Instantly attack the target, causing (200% of Attack power)
-- damage and healing you for 15% of your maximum health.
-- Killing an enemy that yields experience or honor resets
-- the cooldown of Impending Victory.   Has a 30 sec Cooldown and
-- costs 10 rage.   GCD is 1.5 sec.
c.AddSpell("Impending Victory", "for Prot", {
   Melee = true,
   CheckFirst = function()
      return c.HasTalent("Impending Victory") and c.GetCooldown("Impending Victory") == 0
   end,
})

--Increases Current and Maximum health by 30% for 15 sec.
c.AddOptionalSpell("Last Stand", "for Prot", {
   NoGCD = true,
   ShouldHold = function()
      return s.HealthPercent("player") > 70
   end
})

-- Throw a whirling axe at the target location that inflicts (82.5% of Attack power)
-- damage to enemies within 6 yards every 1 sec. Lasts 10 sec.   Has a 60 sec. Cooldown.
c.AddOptionalSpell("Ravager", "for Prot", {
   CheckFirst = function()
      return c.HasTalent("Ravager") and c.GetCooldown("Ravager") == 0
   end,
})

-- Instantly attack an enemy and two additional enemies, dealing (300% of Attack power)
-- damage to the primary target and 50% damage to the secondary targets, and generating 20 Rage.
-- Your successful dodges and parries reset the cooldown on Revenge.
-- Cooldown is 9 sec and GCD = 1.5 sec.
c.AddSpell("Revenge", "for Prot", {
   NotIfActive = true,
   CheckFirst = function()
      return c.GetCooldown("Revenge") == 0
   end,
})

-- Raise our shield absorbing   899 damage for the next 6 seconds.
-- Consumes up to 40 additional Rage to increase the damage taken.
-- Note: WoD allows this on ALL Warriors. Confused how this works
--         with Fury and Arms.
c.AddOptionalSpell("Shield Barrier", "for Prot", {
   NoGCD = true,
   CheckFirst = function(z)
--      return c.IsTanking()
--         and not c.HasBuff("Shield Barrier", true)
--         and not c.HasBuff("Shield Block", true)
--         and not c.InDamageMode()
      if a.Rage > 60
         and not c.HasBuff("Shield Barrier", true)
         and not c.HasBuff("Shield Block", true)
         and not c.InDamageMode() then
            z.FlashSize = s.FlashSizePercent() / 2
      end
      return a.Rage > 60
         and not c.HasBuff("Shield Barrier", true)
         and not c.HasBuff("Shield Block", true)
         and not c.InDamageMode()
   end,
})

-- Raise your shield blocking every melee attack for the next 6 seconds.
-- These can be critical blocks.   Maximum 2 charges.
-- Cost is 60 rage and has a Cooldown of 1.5 seconds
c.AddOptionalSpell("Shield Block", "for Prot", {
   NoGCD = true,
   Melee = true,
   CheckFirst = function()
      return not (c.HasDebuff("Demoralizing Shout")
                   or c.HasBuff("Ravager")
                   or c.HasBuff("Shield Wall")
                   or c.HasBuff("Last Stand")
                   or c.HasBuff("Enraged Regeneration")
                   or c.HasBuff("Shield Block"))
--   CheckFirst = function(z)
--      if not (a.Rage > 60)
--         and not c.HasBuff("Shield Barrier", true)
--         and not c.InDamageMode() then
--            z.FlashSize = s.FlashSizePercent() / 2
--      end
--      return a.Rage > 60
--         and not c.HasBuff("Shield Barrier", true)
--         and not c.InDamageMode()
   end,
})

-- Slam the target with your shield, causing [(Attack power * 0.36)] damage and
-- generating 20 Rage.   Critical strikes with Shield Slam cause your next
-- Heroic Strike to cost no Rage and be a critical strike.
-- Cooldown is 6 sec and GCD is 1.5 sec.
c.AddSpell("Shield Slam", "for Prot", {
   NotIfActive= true,
--   CheckFirst = function()
--      return c.GetCooldown("Shield Slam") == 0
--   end,
})

--Reduces all damage taken by 40% for 8 sec.
c.AddOptionalSpell("Shield Wall", "under 3 min", {
   NoGCD = true,
   Enabled = function()
      return not c.HasGlyph("Shield Wall")
   end,
})

--Reduces all damage taken by 40% for 8 sec.
c.AddOptionalSpell("Shield Wall", "3+ min", {
   NoGCD = true,
   Enabled = function()
      return c.HasGlyph("Shield Wall")
   end,
})

-- Hurls your weapon at an enemy, causing 60% Physical damage and stunning for 4 sec.
-- Deals quadruple damage to targets permanently immune to stuns.
-- Cast is instant with a 30 sec. cooldown.
c.AddSpell("Storm Bolt", "for Prot", {
   CheckFirst = function()
      return c.HasTalent("Storm Bolt") and c.GetCooldown("Storm Bolt") == 0
   end
})

--10122014 rdh: Removed Weakend_Blows and changed cooldown from 6 to 5.54
c.AddSpell("Thunder Clap", "for Debuff", {
   Melee = true,
   CheckFirst = function()
--      local debuffNeeded = not c.HasDebuff(c.WEAKENED_BLOWS_DEBUFFS)
      local damageMode = c.InDamageMode()
      if c.AoE then
--         return debuffNeeded or damageMode
         return damageMode
      else
--         return debuffNeeded and not damageMode
                  return not damageMode
      end
   end,
})

c.AddSpell("Thunder Clap", "for Refresh", {
   Melee = true,
   CheckFirst = function()
--      return c.GetDebuffDuration(c.WEAKENED_BLOWS_DEBUFFS) < 4 or c.AoE
          return c.AoE
   end,
})

c.AddSpell("Delay for Prot Rage Generator", nil, {
   ID = "Battle Shout",
   IsMinDelayDefinition = true,
   GetDelay = function()
      return min(
            c.GetCooldown("Shield Slam", false, 6),
            c.GetCooldown("Revenge", false, 9),
            c.GetCooldown("Battle Shout", false, 60)),
         c.InDamageMode() and .2 or .5
   end,
})

c.AddTaunt("Heroic Throw", nil, {
   NoGCD = true,
   CheckFirst = function()
      return c.GetCooldown("Heroic Throw") == 0
   end
})

-- Throw down a war banner within 30 yards that forces all enemies
-- within 15 yards of the banner to focus attacks on the Warrior for 6 sec.
-- Lasts 30 sec. with a 3 min. Cooldown
-- You can Intervene to your war banner.
c.AddTaunt("Mocking Banner", nil, {
   NoGCD = true,
   CheckFirst = function()
      return c.GetCooldown("Mocking Banner") == 0
   end
})

c.AddTaunt("Taunt", nil, {
   NoGCD = true,
   CheckFirst = function()
      return (c.HasTalent("Vigilance") and c.HasBuff("Vigilance")) or c.GetCooldown("Taunt") == 0
   end,
})
